from django.contrib import admin
from models import GameTeXObject, GameTeXFieldValue, GameTeXUser

class CustomFieldInline(admin.TabularInline):
    model = GameTeXFieldValue

class GTUInline(admin.TabularInline):
    model = GameTeXUser

class GTOAdmin(admin.ModelAdmin):
    fields = ('name', 'macro', 'classes')
    readonly_fields = ('classes',)
    inlines = [CustomFieldInline, GTUInline]

admin.site.register(GameTeXObject, GTOAdmin)


__author__ = 'ternus'

from models import *
import json

def import_gametex(filename, preserve=False):
    """
    Import GameTeX from a file.  Assumes file exists and is sane JSON.
    """
    print "Importing GameTeX from %s." % filename
    if preserve:
        print "Preserving existing database values."
    gametex_file = file(filename, 'r')
    gametex_json = json.load(gametex_file)
    print "Found %d objects." % len(gametex_json)
    for obj in gametex_json:
        print "Loading %s... " % (obj['macro'] if 'macro' in obj else ''),
        g = GTO.objects.get_or_create(macro=obj['macro'])
        if not g[1]:
            print "<found!> ",
        g = g[0]
        if not (g.name and preserve):
            g.name = obj['name']
        g.save()
        print '[',
        for cls in obj['classes']:
            c = GameTeXClass.objects.get_or_create(name=cls)[0]
            g.classes.add(c)
            print cls,
        print '] { ',
        for field in obj:
            if field == 'macro' or field == 'name' or field == 'classes': continue
            f = GameTeXField.objects.get_or_create(name=field)[0]
            v = GameTeXFieldValue.objects.get_or_create(field=f, object=g)
            if v[1] or not preserve:
                v[0].value=obj[field]
                print "%s=%s " % (field, obj[field]),
            else:
                print "%s==%s " % (field, v[0].value),
        print "} done."
    print "Done loading GameTeX objects."

def create_users(force=False):
    """
    Automatically Create users from GameTeX objects.
    """
    print "Creating users..."
    from django.contrib.auth.models import User
    pcs = GTO.bc('PC')

    # sanity check
    fail = False
    for pc in pcs:
        if not pc.has_field('username'):
            print "PC %s (%s) doesn't have username set!" % (pc.name, pc.macro)
            fail = True
        if not pc.has_field('password'):
            print "PC %s (%s) doesn't have password set!" % (pc.name, pc.macro)
            fail = True
    if fail and not force:
        print "Couldn't import -- fix errors first."
        return False
    for pc in pcs:
        if not (pc.has_field('username') and pc.has_field('password')): continue
        u = User.objects.create_user(username=pc.username, password=pc.password)
        GameTeXUser.objects.get_or_create(gto=pc,user=u)
        print "Created %s" % pc.username
    print "Done creating users."


from django.db import models

# Create your models here.

class GameTeXClass(models.Model):
    name = models.CharField(max_length=256, unique=True, primary_key=True)

    def __unicode__(self):
        return self.name

class GameTeXField(models.Model):
    name = models.CharField(max_length=256, unique=True, primary_key=True)

    def __unicode__(self):
        return self.name

class GameTeXFieldValue(models.Model):
    field = models.ForeignKey('GameTeXField')
    object = models.ForeignKey('GameTeXObject')
    value = models.CharField(max_length=256, blank=True)

    def __unicode__(self):
        return "%s %s:%s" % (self.object, self.field, self.value)

class GameTeXObject(models.Model):
    macro = models.CharField(max_length=256, unique=True, primary_key=True)
    name = models.CharField(max_length=256)
    classes = models.ManyToManyField(GameTeXClass)
    custom_fields = models.ManyToManyField(GameTeXField, through=GameTeXFieldValue)

    def __unicode__(self):
        return "%s [%s]" % (self.macro, self.name)

    def __getattr__(self, item):
        """
        Allows access to custom fields through dot syntax
        e.g. gto.market
        """
        return self.field(item)

    def __setattr__(self, key, value):
        try:
            self.set_field(key, value)
        except AttributeError:
            return super(GameTeXObject, self).__setattr__(key, value)

    def has_field(self, field):
        return GameTeXFieldValue.objects.filter(object=self, field=field).exists()

    def field(self, item):
        flds = GameTeXField.objects.filter(name=item)
        if len(flds):
            vals = GameTeXFieldValue.objects.filter(field=flds[0],object=self)
            if len(vals):
                return vals[0].value
        raise AttributeError

    def set_field(self, key, value):
        try:
            v = GameTeXFieldValue.objects.get(field__name=key,object=self)
            v.value = value
            v.save()
        except:
            raise AttributeError

    @classmethod
    def bc(cls, clname):
        return GameTeXObject.objects.filter(classes__name=clname)


class GameTeXUser(models.Model):
    gto = models.ForeignKey('GameTeXObject')
    user = models.ForeignKey('GameTeXUser')

# shortcut
GTO = GameTeXObject


from optparse import make_option
from django.core.management import BaseCommand
import os
from gametex.models import GameTeXObject, GameTeXField, GameTeXFieldValue, GameTeXClass, GameTeXUser
from gametex.loader import import_gametex, create_users
__author__ = 'cternus'

class Command(BaseCommand):
    args = '<path to JSON file> [--clear] [--create-users]'
    help = 'Load data from a GameTeX JSON file.  Updates data in-place unless --clear is passed.'
    option_list = BaseCommand.option_list + (
        make_option('--clear',
                    action='store_true',
                    dest='clear',
                    default=False,
                    help='Delete existing GameTeX objects first.'),
        make_option('--preserve',
                    action='store_true',
                    dest='preserve',
                    default=False,
                    help='If GameTeX data conflicts with database data, preserve the database.'),
        make_option('--create-users',
                    action='store_true',
                    dest='create-users',
                    default=False,
                    help='Create Django users and GameTeXUser models for all PCs.'),
        make_option('--force-create',
                    action='store_true',
                    dest='force-create',
                    default=False,
                    help='Create all valid users even if invalid entries exist.'),
        )

    def handle(self, *args, **options):
        print args
        if options['clear']:
            GameTeXFieldValue.objects.all().delete()
            GameTeXField.objects.all().delete()
            GameTeXObject.objects.all().delete()
            GameTeXClass.objects.all().delete()
            GameTeXUser.objects.all().delete()
        if len(args) != 1:
            print 'You must provide the name of a JSON file.'
            return
        if not os.path.exists(args[0]):
            print 'That file doesn\'t exist!'
            return

        import_gametex(args[0], preserve=options['preserve'])
        if options['create-users']:
            create_users(force=options['force-create'])
        print "Done! %s fully loaded." % args[0]


from distutils.core import setup

setup(
    name='gametex-django',
    version='0.2.0',
    packages=['gametex','gametex.management.commands'],
    author='Christian Ternus',
    author_email='ternus@cternus.net',
    url='http://github.com/ternus/gametex-django',
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    description='Django integration for GameTeX',
    long_description=open('README.txt').read(),
)
